ad_library {

	Library for Mash Up : GeoIP (hostip.info and yahoo maps)

    @author Hamilton Chua (ham@solutiongrove.com)
    @creation-date 2006-03-20
}

namespace eval mashup::geoip { }

# ******* GEO IP ***********

ad_proc -public mashup::geoip::hostip_url {

} {
	Returns the ip address from where we will get ip and position info
} {
	return "http://api.hostip.info/"
}

ad_proc -private mashup::geoip::connect {
	-ip
} {
	Fetches the xml from hostip. 
	Returns an xml node from which we can extract the information we need.
} {
	set url [mashup::geoip::hostip_url]
	append url "?ip=$ip&position=true"
	set doc [util_httpget $url]
	set xml [xml_parse -persist $doc]
	set root [xml_doc_get_first_node $xml]
	set featuremember [xml_node_get_children_by_name $root "gml:featureMember"]
	set hostipinfo [xml_node_get_children_by_name $featuremember "Hostip"]
	return ${hostipinfo}
}

ad_proc -private mashup::geoip::getinfo {
	-xmlnode
	{-info "gml:name"}
} {
	Accepts an xmlnode as generated by mashup::geoip::connect. 
	Pass the appropriate parameter to retrieve the data from the node.	
	By default, this will return the gml:name
	other values for info are
   	- countryName
	- countryAbbrev
} {
	return "[xml_node_get_content [xml_node_get_first_child_by_name $xmlnode $info]]"
}

ad_proc -private mashup::geoip::getlocation {
	-xmlnode
	{-position "longitude"}
} {
	Retrieve the longitude and latitude from the xmlnode generated by mashup::geoip::getinfo.
	Specify a position longitude or latitude, defualt is longitude.
} {
	set iplocation [xml_node_get_children_by_name $xmlnode "ipLocation"]
	if { ![catch {set pointproperty [xml_node_get_children_by_name $iplocation "gml:PointProperty"]}] } {
		set point [xml_node_get_children_by_name $pointproperty "gml:Point"]
		set coordinates [xml_node_get_content [xml_node_get_first_child_by_name $point "gml:coordinates"]]
		set loclist [split $coordinates ","]
		if { $position == "longitude" } {
			return [lindex $loclist 0]
		} 
		if { $position == "latitude" } {
			return [lindex $loclist 1]
		}
	}
	return ""
}

ad_proc -public mashup::geoip::getflag {
	-ip
	{-alt "IP Address Lookup"}
} {
	Returns an html image with the country derived from the ip address info.
} {
	set api_url [mashup::geoip::hostip_url]
	return "<img src=\"${api_url}flag.php?ip=$ip\" alt=\"${alt}\" border=0>"
}

ad_proc -public mashup::geoip::recordvisit {
	{ -ip "" }
	{-user_id ""}
	{-norecord:boolean}
} {
	Record a visit to the site.
	If norecord is passed we just return the geoip id or create a new one but we don't
		record the time/date of visit
} {
	if { ![exists_and_not_null ip] } { set ip [ns_conn peeraddr] }
	if { ![exists_and_not_null user_id] } { set user_id [ad_conn user_id] }
	if { [exists_and_not_null ip] } {
		# check if ip is already in db
		if { ![db_0or1row "ipexists" "select id, longitude from mashup_geoip where ip = :ip"] } {
		# no
			# retrieve info from hostip
			set hostinfo [mashup::geoip::connect -ip $ip]
			set name [mashup::geoip::getinfo -xmlnode $hostinfo -info "gml:name"]
			set country [mashup::geoip::getinfo -xmlnode $hostinfo -info "countryName"]
			set abbrev [mashup::geoip::getinfo -xmlnode $hostinfo -info "countryAbbrev"]
			set lat [mashup::geoip::getlocation -xmlnode $hostinfo -position "latitude"]
			set long [mashup::geoip::getlocation -xmlnode $hostinfo -position "longitude"]			
			# store
			set id [db_nextval "mashup_geoip_id"]
			db_dml "newgeoip" "insert into mashup_geoip (id,ip,city,countryname,countryabbrev,longitude,latitude) values (:id,:ip,:name,:country,:abbrev,:long,:lat)"
		} else {
		# yes
			# mashup_geoip_id has been retrieved
			# let's check if the ip has been updated by hostip
			if { ![exists_and_not_null longitude]} {
				# retrieve info from hostip
				set hostinfo [mashup::geoip::connect -ip $ip]
				set name [mashup::geoip::getinfo -xmlnode $hostinfo -info "gml:name"]
				set lat [mashup::geoip::getlocation -xmlnode $hostinfo -position "latitude"]
				set long [mashup::geoip::getlocation -xmlnode $hostinfo -position "longitude"]			
				# update the record
				db_dml "updategeoip" "update mashup_geoip set city=:name, longitude=:long, latitude=:lat where id=:id"
			}
		}
		# record visit
		if { !$norecord_p } {	
			db_dml "recordvisit" "insert into mashup_geoip_visits (mashup_geoip_id,user_id) values (:id,:user_id)"
		}
		return $id
	}
	return ""
}